/*
Copyright (C) 2009 - Paolo Pagano and Claudio Salvadori
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/**
 * @file   DAQSData.h
 * @author Paolo Pagano and Claudio Salvadori
 * @date   22-08-2009
 * 
 * @brief  This file contains information on the DAQ 
 * classes*/

#ifndef DAQSDATA_H
#define DAQSDATA_H

#include "TObject.h"
#include "TClonesArray.h"
#include "TRefArray.h"
#include "TRef.h"
#include "TH1.h"
#include "TMath.h"
#include <vector>

#include "Data_struct.h"
#include "DAQPayload.h"
//////////////////////////////////////////////////////////////////////////
//                                                                      //
// DAQSData                                                             //
//                                                                      //
// Description of the event and DAQSDatas parameters                    //
//                                                                      //
//////////////////////////////////////////////////////////////////////////

/** \class DAQSData
*\brief Class containing members and methods for the data acquired through the link
*\warning This class has to be modified everytime the xml file payload description is changed
*/
class DAQSData : public TObject {

private:
  	DAQPayload 	daqp;		/**< Member raw data*/
public:
	DAQSData();
	DAQSData(const DAQSData& orig);
	DAQSData(std::vector<SData> sdata_);

	virtual 	~DAQSData() {Clear();}

	void 		Clear(Option_t *option="");
	
	/** 
	* \brief returns the raw data
	*/  
	DAQPayload*	getPayload(void) { return &daqp; }
	
	/** 
	* \brief prints the data as a table
	*/   
	void printData() const;

	ClassDef(DAQSData,1)  
};

#endif
