/*
  Copyright (C) 2009 -  Claudio Salvadori
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/**
 *\file CSerial.cpp
 *\brief  Serial access functions development
 */

#include "CSerial.h"

#define MAX_ACK_LENGHT 100

using namespace std;

CSerial::~CSerial(){
	Serial_close();
}

s_ret CSerial::Serial_handshaking(handshake_style style){
	s_ret help;
	unsigned char data[MAX_ACK_LENGHT];
	
	//handshaking if it exists...
	switch(style){
	case H_S_STANDARD: // request -> response -> parsing (on demand)
		fprintf(stderr, "Start H_S_STANDARD Function %s Line %d ", __FUNCTION__, __LINE__);
		help = Serial_write((unsigned char*)(*se_Handshk)[0].h_Pkt, (*se_Handshk)[0].h_Size_P);
		if(help != S_RET_OK)
			return help;
		break;
		
	case H_S_NONE: // no handshaking /todo SBAGLIATO
		return S_RET_OK;
		break;
		
	case H_S_START: // request -> response -> parsing (continuous streaming)
		fprintf(stderr,"Start H_S_START Function %s Line %d\n ", __FUNCTION__, __LINE__);
		for(unsigned int i = 0; i < se_Handshk->size(); i++){
			//send handshking message
			help = Serial_write((unsigned char*)(*se_Handshk)[i].h_Pkt, 
					    (*se_Handshk)[i].h_Size_P);
			if(help != S_RET_OK)
				return help;

			if(((*se_Handshk)[i].h_Size_A != 0) && ((*se_Handshk)[i].h_Size_N != 0)){
							
				//reading answer
				help = Serial_read(data, (*se_Handshk)[i].h_Size_A);
				if(help != S_RET_OK)
					return help;

				if(memcmp((*se_Handshk)[i].h_Ack, data, (*se_Handshk)[i].h_Size_A) != 0){
					if(memcmp((*se_Handshk)[i].h_Nack, data, (*se_Handshk)[i].h_Size_N) == 0)
						return S_S_NACK;
					else 
						return S_S_GEN_ERR;
				}
			}
		}
		break;
	}
	
	return S_RET_OK;
}
	
s_ret CSerial::getDefinition (struct SDef def, s_read_style sty){
	
	se_Name = (char *) def.s_Name.c_str();		
	se_Chan = def.s_Chan;
	se_Style = sty;
	
	switch(se_Style){
	case S_STYLE_LENGTH:
		se_Size = def.s_Size;
#ifdef DEBUG
		printf("Lenght style\n");
#endif
		break;
	
	case S_STYLE_S_E:
		se_Size = def.s_Size;
		se_Start = (unsigned char)def.s_Option->o_Start;
		se_End = (unsigned char)def.s_Option->o_End;
#ifdef DEBUG
		printf("Start/end style \n");
#endif
		break;
	}
	
	if(def.s_Preset == NULL){
		return S_S_PRESET_MISS_ERR;
	}else{
		se_Baud = def.s_Preset->rs_Baud;
			se_DBits = def.s_Preset->rs_DBits;	
				se_Parity = def.s_Preset->rs_Parity; 	
				se_StopB = def.s_Preset->rs_StopB;
					
					se_Handshk = def.s_Handshk;
	}
	
	se_HStyle = def.s_Style;

	if(se_HStyle == H_S_START){
		s_ret help;
		help = Serial_init();
		if(help != S_RET_OK)
			return help;
	
		help = Serial_handshaking(se_HStyle);
	
		//Handshake did! disable all the eventual future possibilities to have an hanshake!
		se_HStyle = H_S_NONE;
	
		return help;
	}else{
		return Serial_init();
	}

}

//Blocking function
s_ret CSerial::retStream(unsigned char **buffer, unsigned long *length){
	s_ret help;
	
	switch(se_Style){		
	case S_STYLE_LENGTH:
		if(se_HStyle == H_S_STANDARD){
			help = Serial_handshaking(se_HStyle);
			if(help != S_RET_OK)
				return help;
		}
		
		*buffer = new unsigned char[se_Size];
		help = Serial_read(*buffer, (unsigned int) se_Size);
		*length = se_Size_r;
#ifdef DEBUG
		for(unsigned long i = 0; i < *length; i++)
			printf("Character received: %x, %d\n", (int)(*buffer)[i], (int)(*buffer)[i]);
#endif
		return help;
		break;
		
	case S_STYLE_S_E:
		std::vector<unsigned char> buff;
		unsigned char buff_t;
		unsigned int size = 1;
		
		if(se_HStyle == H_S_STANDARD){
			help = Serial_handshaking(se_HStyle);
			if(help != S_RET_OK)
				return help;
		}
		
		//Searching for syncro character (start character)...
		while(buff_t != se_Start){				
			help = Serial_read(&buff_t, size);
			if(help != S_RET_OK)
				return help;
		}
		
		//clear the buffer
		buff.clear();

		//... and reading untill end character
		while(1){
			help = Serial_read(&buff_t, size);
			if(help != S_RET_OK)
				return help;

			//End character found and the coerent packet size	
			if((buff_t == se_End) && (buff.size() >= se_Size)){
#ifdef DEBUG
				cerr << "Buffer size: " << buff.size() << " Correct size: " << se_Size << endl;
				cerr << "Total packet : " << received << " Uncorrect packet: " << lost << endl;
#endif
				*length = buff.size();
				*buffer = new unsigned char[*length];
				
				for(unsigned long i_s = 0; i_s < *length; i_s++) 
					(*buffer)[i_s] = buff[i_s];
				
#ifdef DEBUG
				printf("Buffer:");
				for(unsigned long i_s = 0; i_s < *length; i_s++){
					printf(" %d", (*buffer)[i_s]);
				}
				printf("\n");
#endif
				return S_RET_OK;
			}
	
			//push back the character
			buff.push_back(buff_t);
		};
		break;
	}

	return S_RET_OK;
}

s_ret CSerial::Serial_init(void){
#if defined(__WIN32__) || defined(__CYGWIN__) /* WIN32 Implementation */
	
#ifdef DEBUG
	printf("Serial name: %s\n", se_Name);
#endif

	handlerCom = CreateFile(se_Name, 
				GENERIC_READ | GENERIC_WRITE,
				0, // exclusive access
				NULL, // no security
				OPEN_EXISTING,
				0, // no overlapped I/O -> blocking i/o
				NULL
				); // null template 

	if (handlerCom == INVALID_HANDLE_VALUE)
		return S_S_OPEN_ERR;
 
	bPortReady = SetupComm(handlerCom, 128, 128); // set buffer sizes
 
	bPortReady = GetCommState(handlerCom, &ctrl);
#ifdef DEBUG
	printf("Baud rate: %d\n", se_Baud);
#endif
	ctrl.BaudRate = se_Baud;
	ctrl.ByteSize = se_DBits;
	
	if(strcmp(se_Parity.c_str(), "odd") == 0){
		ctrl.Parity = ODDPARITY;
	}else if(strcmp(se_Parity.c_str(), "even") == 0){
		ctrl.Parity = EVENPARITY;
	}else if(strcmp(se_Parity.c_str(), "none") == 0){
		ctrl.Parity = NOPARITY;
	}
	
	if(se_StopB==1){
		ctrl.StopBits = ONESTOPBIT;
	}else if(se_StopB==1.5){
		ctrl.StopBits = ONE5STOPBITS;
	}else if(se_StopB==2){
		ctrl.StopBits = TWOSTOPBITS;
	}
	
	ctrl.fAbortOnError = TRUE;
 
	// set XON/XOFF
	ctrl.fOutX = FALSE; // XON/XOFF off for transmit
	ctrl.fInX = FALSE; // XON/XOFF off for receive
	// set RTSCTS
	ctrl.fOutxCtsFlow = FALSE; // turn on CTS flow control
	ctrl.fRtsControl = RTS_CONTROL_DISABLE; // 
	// set DSRDTR
	ctrl.fOutxDsrFlow = FALSE; // turn on DSR flow control
	ctrl.fDtrControl = DTR_CONTROL_DISABLE; // 
 
	bPortReady = SetCommState(handlerCom, &ctrl);
 
	// Communication timeouts are optional
 
	bPortReady = GetCommTimeouts (handlerCom, &CommTimeouts);
 
	CommTimeouts.ReadIntervalTimeout = 10000;
	CommTimeouts.ReadTotalTimeoutConstant = 10000;
	CommTimeouts.ReadTotalTimeoutMultiplier = 10000;
	CommTimeouts.WriteTotalTimeoutConstant = 10000;
	CommTimeouts.WriteTotalTimeoutMultiplier = 10000;
 
	bPortReady = SetCommTimeouts (handlerCom, &CommTimeouts);

	if(bPortReady == FALSE)
		return S_S_OPEN_ERR;

	return S_RET_OK;
	
#else 									/* Unix-like Implementation */

	speed_t baud;

	fd = open(se_Name, 
		  O_RDWR 		// Read-Write I/O
		  | O_NOCTTY 		// Refers to device terminal
		  | O_SYNC		// Blocking I/O 
		  ); 
	fprintf(stderr, "We opened serial on descriptor %d\n", fd);

	if (fd == -1)    //da ripristinare
		return S_S_OPEN_ERR;
	else
		fcntl(fd, F_SETFL, 0);

	// Get the current options for the port...
	tcgetattr(fd, &options);
	
	//Reset option...
	options.c_cflag = 0x00;
	options.c_iflag = 0x00;
	options.c_oflag = 0x00;
	options.c_lflag = 0x00;
	options.c_line  = 0x00;

	//Set the correct option for the port...
	options.c_cflag = CREAD | CLOCAL;

	// Set the baud rates to...
	switch(se_Baud)
	{
	case 9600:
		baud = B9600;	//0000015
		break;
	case 19200:
		baud = B115200;	//0000016
		break;
	case 38400:
		baud = B38400; 	//0000017
		break;
	case 57600:
		baud = B57600; 	//0010001
		break;
	case 115200:
		baud = B115200; //0010002
		break;
	default:
		return S_S_OPEN_ERR;
		break;
	}

	options.c_cflag |= baud;

	cfsetispeed(&options, baud);
	cfsetospeed(&options, baud);

	// Set numeber of data bits to...
	switch(se_DBits)
	{
	case 5:
		options.c_cflag |= CS5;
		break;
        case 6:
		options.c_cflag |= CS6;
		break;
       	case 7:
		options.c_cflag |= CS7;
		break;
        case 8:
		options.c_cflag |= CS8;
		break;
	default:
		return S_S_OPEN_ERR;
		break;
	}
	
	//Parity setting...
	if(strcmp(se_Parity.c_str(), "odd") == 0){
		options.c_cflag |= (PARENB | PARODD);
	}else if(strcmp(se_Parity.c_str(), "even") == 0){
		options.c_cflag |= PARENB;
		options.c_cflag &= ~PARODD;
	}else if(strcmp(se_Parity.c_str(), "none") == 0){
		options.c_cflag &= (~PARENB | ~PARODD);
	}

	//Stop Bits setting...
	if(se_StopB == 1){
		options.c_cflag &= ~CSTOPB;
	}else if(se_StopB == 1.5){
		fprintf(stderr,"Stop Bit  = 1.5 : value not permitted on Linux");
		return S_S_OPEN_ERR;
	}else if(se_StopB == 2){
		options.c_cflag |= CSTOPB;
	}

	options.c_iflag |= IGNBRK; //Ignore BREAK condition on input. 

	options.c_oflag |= OPOST; //Enable implementation-defined output processing.
                                                    
	options.c_cc[VTIME]    = 0;     /* inter-character timer unused */            
	options.c_cc[VMIN]     = 1;     /* blocking read until data_size character arrives */ 

#ifdef DEBUG
	fprintf(stderr, "CFLAGS: %x\n", options.c_cflag);
	fprintf(stderr, "IFLAGS: %x\n", options.c_iflag);
	fprintf(stderr, "OFLAGS: %x\n", options.c_oflag);
	fprintf(stderr, "LFLAGS: %x\n", options.c_lflag);
#endif

	tcsetattr(fd, TCSANOW, &options);

	return S_RET_OK;

#endif
}

void CSerial::Serial_close(void){
#if defined(__WIN32__) || defined(__CYGWIN__) 	/* WIN32 Implementation */
	CloseHandle(handlerCom);
#else						/* Unix-like Implementation */
	close(fd);
#endif
}

s_ret CSerial::Serial_read(unsigned char *data, unsigned int data_size){
	fprintf(stderr, "We are waiting for %d bytes\n", data_size);
#if defined(__WIN32__) || defined(__CYGWIN__) 	/* WIN32 Implementation */
	int i = 0;
	DWORD read_size;
	if( ReadFile(handlerCom, data, sizeof(char) * data_size, &read_size, NULL) == FALSE )
		return S_S_READ_ERR;
	else{	
		se_Size_r = (unsigned int)read_size;
		if((unsigned int)read_size == data_size){
			fprintf(stderr, "%d bytes read\n", read_size);
			fprintf(stderr, "String content: ");
			for(i = 0; i < data_size; i++)
				fprintf(stderr,"%x ", *(data + i));
			return S_RET_OK;
		}else{
			if(read_size < 0)
				return S_S_P_CHAR_ERR;
			else if(read_size == 0)
				return S_S_R_TIMEOUT_ERR;
			else return S_S_LENGHT_ERR;
		}
	}	
#else 		 				/* Unix-like Implementation */
// 	tcsetattr(fd, TCSANOW, &options);
	int remaining_bytes = data_size;
	unsigned char * pointer = data;
	while ( remaining_bytes > 0 ){
		int n = read(fd, pointer, 1);
#ifdef DEBUG
		fprintf(stderr, "%x", *pointer);
#endif
		pointer += n;
		remaining_bytes -= n;
		
		if(n < 0)
			return S_S_P_CHAR_ERR;
	
		if(n == 0)
			return S_S_R_TIMEOUT_ERR;
	}
	se_Size_r = data_size;
	return S_RET_OK;

#endif
}

s_ret CSerial::Serial_write(unsigned char* data, unsigned int data_size){
#ifdef DEBUG
	int i;
	fprintf(stderr,"Packet to send (size): %d\n", data_size);
	fprintf(stderr,"Packet to send: ");
	for(i = 0; i < data_size; i++)
		fprintf(stderr,"%d ", *(data + i));
	fprintf(stderr,"\n");
#endif	

#if defined(__WIN32__) || defined(__CYGWIN__) /* WIN32 Implementation */
	DWORD write_size;
	if( !WriteFile(handlerCom, data, sizeof(char)*data_size, &write_size, NULL) )	
		return S_S_WRITE_ERR;
	else{
		if((unsigned int)write_size == data_size){
			fprintf(stderr, "written %d bytes\n", write_size);
			return S_RET_OK;
		}else{
			if(write_size < 0)
				return S_S_G_CHAR_ERR;
			else if(write_size == 0)
				return S_S_T_TIMEOUT_ERR;
			else return S_S_LENGHT_ERR;
		}
	}
	
#else 										/* Unix-like Implementation */
		// 	int n = write(fd, data, sizeof( unsigned int )*data_size);
	int n = write(fd, data, data_size);

	#ifdef DEBUG
	data[ data_size ] = 0x0;
	fprintf(stderr, "written %d bytes\n", n);
	#endif

	if(n < 0)
		return S_S_G_CHAR_ERR;
	
	if(n == 0)
		return S_S_T_TIMEOUT_ERR;
	
	if((unsigned int)n != data_size){
		return S_S_LENGHT_ERR;
	}
	
	return S_RET_OK;

#endif
}

